/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 *
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2007 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  上記著作権者は，Free Software Foundation によって公表されている 
 *  GNU General Public License の Version 2 に記述されている条件か，以
 *  下の(1)～(4)の条件を満たす場合に限り，本ソフトウェア（本ソフトウェ
 *  アを改変したものを含む．以下同じ）を使用・複製・改変・再配布（以下，
 *  利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを再利用可能なバイナリコード（リロケータブルオブ
 *      ジェクトファイルやライブラリなど）の形で利用する場合には，利用
 *      に伴うドキュメント（利用者マニュアルなど）に，上記の著作権表示，
 *      この利用条件および下記の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを再利用不可能なバイナリコードの形または機器に組
 *      み込んだ形で利用する場合には，次のいずれかの条件を満たすこと．
 *    (a) 利用に伴うドキュメント（利用者マニュアルなど）に，上記の著作
 *        権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 利用の形態を，別に定める方法によって，上記著作権者に報告する
 *        こと．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者を免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者は，
 *  本ソフトウェアに関して，その適用可能性も含めて，いかなる保証も行わ
 *  ない．また，本ソフトウェアの利用により直接的または間接的に生じたい
 *  かなる損害に関しても，その責任を負わない．
 * 
 */

/*  
 *  2014/01:
 *		Modified for STM32F0-Discovery(STM32F051) by dinau.  
 *
 *  */

#ifndef TOPPERS_CQ_STARM_H
#define TOPPERS_CQ_STARM_H

#include <sil.h>

/*
 *  CORTEX-M3 CPUの依存部のインクルード
 */
#include "arm_m_gcc/arm_m.h"

/*
 *  システムクロックの定義（発振子周波数8MHz）
 */
//#define HSE_CLOCK		(8000000)
#define HSI_CLOCK		(8000000)
#define SYS_CLOCK		(HSI_CLOCK/2 * 12)
#define PCLK_CLOCK		(SYS_CLOCK)

/*
 *  割込み番号の最大値
 */
#define TMAX_INTNO   (16 + 30)

/*
 *  割込み優先度のビット幅
 */
#define TBITW_IPRI     4

/*
 *  割込み優先度ビット幅中のサブ優先度のビット幅
 */
#define TBITW_SUBIPRI  0

/*
 *  割込みベクタ番号定義
 */
#define IRQ_VECTOR_USART1		(16 + 27)
#define IRQ_VECTOR_USART2		(16 + 28)
//#define IRQ_VECTOR_TIM4			(16 + 30)


/* STM32F10Xのペリフェラルレジスタ定義 */
#define PERIPH_REG_BASE			(0x40000000UL)
#define SRAM_BASE				(0x20000000UL)

#define APB1_PERIPH				(PERIPH_REG_BASE)
#define APB2_PERIPH				(PERIPH_REG_BASE + 0x00010000)
#define AHB1_PERIPH				(PERIPH_REG_BASE + 0x00020000)
#define AHB2_PERIPH				(PERIPH_REG_BASE + 0x08000000)

/* BUS:APB:1 */
#define TIM2_BASE				(APB1_PERIPH)
#define TIM3_BASE				(APB1_PERIPH + 0x0400)
//#define TIM4_BASE				(APB1_PERIPH + 0x0800)
#define TIM6_BASE				(APB1_PERIPH + 0x1000)
#define TIM14_BASE				(APB1_PERIPH + 0x2000)
#define RTC_BASE				(APB1_PERIPH + 0x2800)
#define WWDG_BASE				(APB1_PERIPH + 0x2C00)
#define IWDG_BASE				(APB1_PERIPH + 0x3000)
#define SPI2_BASE				(APB1_PERIPH + 0x3800)
#define USART2_BASE				(APB1_PERIPH + 0x4400)
//#define USART3_BASE			(APB1_PERIPH + 0x4800)
#define I2C1_BASE				(APB1_PERIPH + 0x5400)
#define I2C2_BASE				(APB1_PERIPH + 0x5800)
//#define USB_BASE				(APB1_PERIPH + 0x5C00)
//#define CAN_BASE				(APB1_PERIPH + 0x6400)
//#define BKP_BASE				(APB1_PERIPH + 0x6C00)
#define PWR_BASE				(APB1_PERIPH + 0x7000)
#define DAC_BASE				(APB1_PERIPH + 0x7400)
#define CEC_BASE				(APB1_PERIPH + 0x7800)

/* BUS:APB:2 */
#define SYSCFG_BASE				(APB2_PERIPH)
#define EXTI_BASE				(APB2_PERIPH + 0x0400)
#define ADC1_BASE				(APB2_PERIPH + 0x2400)
//#define ADC2_BASE				(APB2_PERIPH + 0x2800)
#define TIM1_BASE				(APB2_PERIPH + 0x2C00)
#define SPI1_BASE				(APB2_PERIPH + 0x3000)
#define USART1_BASE				(APB2_PERIPH + 0x3800)
#define TIM15_BASE				(APB2_PERIPH + 0x4000)
#define TIM16_BASE				(APB2_PERIPH + 0x4400)
#define TIM17_BASE				(APB2_PERIPH + 0x4800)
#define DBGMCU_BASE				(APB2_PERIPH + 0x5800)

/* BUS:AHB2 */
#define GPIOA_BASE				(AHB2_PERIPH )
#define GPIOB_BASE				(AHB2_PERIPH + 0x0400)
#define GPIOC_BASE				(AHB2_PERIPH + 0x0800)
#define GPIOD_BASE				(AHB2_PERIPH + 0x0C00)
#define GPIOF_BASE				(AHB2_PERIPH + 0x1400)

/* BUS:AHB1 */
#define DMA_BASE				(AHB1_PERIPH)
#define RCC_BASE				(AHB1_PERIPH + 0x1000)
#define FLASH_BASE				(AHB1_PERIPH + 0x2000)
#define CRC_BASE				(AHB1_PERIPH + 0x3000)
#define TSC_BASE				(AHB1_PERIPH + 0x4000)

/* System Control space */
//#define SCS_BASE				(0xE000E000)
//#define SYSTM_BASE			(SCS_BASE + 0x0010)
//#define NVIC_BASE				(SCS_BASE + 0x0100)
//#define SYSCB_BASE			(SCS_BASE + 0x0D00)

/* RCC */
#define RCC_CR					(RCC_BASE)
#define RCC_CFGR				(RCC_BASE + 0x04)
#define RCC_CIR					(RCC_BASE + 0x08)
#define RCC_APB2RSTR			(RCC_BASE + 0x0C)
#define RCC_APB1RSTR			(RCC_BASE + 0x10)
#define RCC_AHBENR				(RCC_BASE + 0x14)
#define RCC_APB2ENR				(RCC_BASE + 0x18)
#define RCC_APB1ENR				(RCC_BASE + 0x1C)
#define RCC_BDCR				(RCC_BASE + 0x20)
#define RCC_CSR					(RCC_BASE + 0x24)


/* NVIC */
#define NVIC_ENAVLE_REG(ch)		(NVIC_BASE + ((ch) >> 5))
#define NVIC_DISABLE_REG(ch)	(NVIC_BASE + 0x80 + ((ch) >> 5))
#define NVIC_SET_PEND_REG(ch)	(NVIC_BASE + 0x100 + ((ch) >> 5))
#define NVIC_CLEAR_PEND_REG(ch)	(NVIC_BASE + 0x180 + ((ch) >> 5))
#define NVIC_ACTIVE_REG(ch)		(NVIC_BASE + 0x200 + ((ch) >> 5))
#define NVIC_PRIO_REG(ch)		(NVIC_BASE + 0x300 + ((ch) >> 2))

/* GPIOx */
#define GPIO_MODER(x)			(x)
#define GPIO_OTYPER(x)			((x) + 0x04)
#define GPIO_OSPEEDR(x)			((x) + 0x08)
#define GPIO_PUPDR(x)			((x) + 0x0C)
#define GPIO_IDR(x)				((x) + 0x10)
#define GPIO_ODR(x)				((x) + 0x14)
#define GPIO_BSRR(x)			((x) + 0x18)
#define GPIO_LCKR(x)			((x) + 0x1C)
#define GPIO_AFRL(x)			((x) + 0x20)
#define GPIO_AFRH(x)			((x) + 0x24)
#define GPIO_BRR(x)				((x) + 0x28)

/* AFIO */
//#define AFIO_EVCR				(AFIO_BASE)
//#define AFIO_MAPR				(AFIO_BASE + 0x04)
//#define AFIO_EXTICR1			(AFIO_BASE + 0x08)
//#define AFIO_EXTICR2			(AFIO_BASE + 0x0C)
//#define AFIO_EXTICR3			(AFIO_BASE + 0x10)
//#define AFIO_EXTICR4			(AFIO_BASE + 0x14)

/* FLASH */
#define FLASH_ACR				(FLASH_BASE)

/* RCCレジスタ定義 */
#define CR_PLL_RDY				(0x02000000)
#define CR_PLL_ON				(0x01000000)
#define CR_HSE_RDY				(0x00020000)
#define CR_HSE_ON				(0x00010000)
#define CR_HSI_RDY				(0x00000002)
//#define CR_HSI_ON				(0x00000001)
#define CFGR_PLLMUL_MASK		(0x003C0000)
#define CFGR_PLL_XTPRE			(0x00020000)
#define CFGR_PLL_SRC			(0x00010000)
#define CFGR_HPRE_MASK			(0x000000F0)
//#define CFGR_PPRE2_MASK		(0x00003800)
#define CFGR_PPRE_MASK			(0x00000700)
#define CFGR_SWS_MASK			(0x0000000C)
#define CFGR_SW_MASK			(0x00000003)
#define CFGR_SW_PLL				(0x02)
#define APB1ENR_USART2_EN		(0x00020000)
/*   */
#define PPRE_DIV1				(0b000 << 8)
#define PPRE_DIV2				(0b100 << 8)
#define	PLLMUL_X6				(0b0100<<18)
#define	PLLMUL_X12				(0b1010<<18)

/* I/O enable */
//#define APB2ENR_ADC3_EN		(0x8000)
#define APB2ENR_USART1_EN		(0x4000)
//#define APB2ENR_TIM8_EN		(0x2000)
#define APB2ENR_SPI1_EN			(0x1000)
#define APB2ENR_TIM1_EN			(0x0800)
//#define APB2ENR_ADC2_EN		(0x0400)
//#define APB2ENR_ADC1_EN		(0x0200)
//#define APB2ENR_IOPG_EN		(0x0100)
#define AHBENR_IOPA_EN			(0x00020000)
#define AHBENR_IOPB_EN			(0x00040000)
#define AHBENR_IOPC_EN			(0x00080000)
#define AHBENR_IOPD_EN			(0x00100000)
#define AHBENR_IOPF_EN			(0x00400000)
//#define APB2ENR_AFIO_EN		(0x0001)
//#define AHBENR_SDIO_EN		(0x0400)
//#define AHBENR_FSMC_EN		(0x0100)
#define AHBENR_CRCE_EN			(0x0040)
#define AHBENR_FLITF_EN			(0x0010)
#define AHBENR_SRAM_EN			(0x0004)
#define AHBENR_DMA_EN			(0x0001)

/* FLASHレジスタ定義 */
#define ACR_LATENCY_MASK		(0x07)
#define ACR_LATENCY_ZERO		(0x00)
#define ACR_LATENCY_ONE			(0x01)
#define ACR_LATENCY_TWO			(0x02)

/* GPIOxレジスタ定義 */
//#define CNF_IN_ANALOG			(0x00)
//#define CNF_IN_FLOATING		(0x01)
//#define CNF_IN_PULL			(0x02)
//#define CNF_OUT_GP_PP			(0x00)
//#define CNF_OUT_GP_OD			(0x01)
//#define CNF_OUT_AF_PP			(0x02)
//#define CNF_OUT_AF_OD			(0x03)
#define MODE_INPUT				(0x00)
#define MODE_OUTPUT				(0x01)
#define	MODE_ALTERNATE			(0x02)
#define	MODE_ANALOG				(0x03)

#define MODE_OUTPUT_2MHZ		(0x00)
#define MODE_OUTPUT_10MHZ		(0x01)
#define MODE_OUTPUT_50MHZ		(0x03)

#define NO_PUPD					(0x00)
#define	PULL_UP					(0x01)
#define	PULL_DOWN				(0x02)

#define	AFR_MODE_MASK(x)		(0x0F << ((x) << 2))
#define	AFR_MODE(x,v)			(((v)&0x0F) << ((x) << 2))
#define	AFR_AF0					(0x000)
#define	AFR_AF1					(0x001)
#define	AFR_AF2					(0x002)
#define	AFR_AF3					(0x003)
#define	AFR_AF4					(0x004)
#define	AFR_AF5					(0x005)
#define	AFR_AF6					(0x006)
#define	AFR_AF7					(0x007)

#define CR_MODE_MASK(p)			(0x03 << ((p) << 1))
#define CR_MODE(p,v)			(((v) & 0x03) << ((p) << 1))
//#define CR_CNF_MASK(x)		(0x0C << ((x) << 2))
//#define CR_CNF(x,v)			((((v) << 2) & 0x0C) << ((x) << 2))

/* AFIOレジスタ定義 */
//#define MAPR_USART2_REMAP		(0x0008)

#ifndef TOPPERS_MACRO_ONLY

#endif /* TOPPERS_MACRO_ONLY */
#endif /* TOPPERS_CQ_STARM_H */
